# -*- coding=utf8 -*-
"""语法解析"""
from .exc import ProtoGrammarError
from .objects import (
    MessageField,
    ServiceMethod,
    MapField,
    field_map,
    Service,
    Message,
    Protobuf,
)


class FieldType(object):
    """field类型"""
    BASE = 1
    MAP = 2
    REF = 3


class HeaderType(object):
    """header类型"""
    SYNTAX = 'syntax'
    PACKAGE = 'package'


def p_error(p):
    if p is None:
        raise ProtoGrammarError('grammar error at EOF')
    raise ProtoGrammarError('grammar error: {} at line {}'.format(
        p.value, p.lineno))


def p_start(p):
    """start : header definition"""
    package, syntax = '', 'proto3'
    for item in p[1]:
        if item[0] == HeaderType.SYNTAX:
            syntax = item[1]
        elif item[0] == HeaderType.PACKAGE:
            package = item[1]

    proto = Protobuf(package, syntax)
    symbols = set()
    for item in p[2]:
        item_name = item.name
        if item_name in symbols:
            raise ProtoGrammarError(
                'grammar error: symbol {} is already defined at line {}'.format(
                    item_name, p.lineno))

        if isinstance(item, Service):
            proto.services[item_name] = item
        elif isinstance(item, Message):
            proto.messages[item_name] = item
        symbols.add(item_name)

    _check_methods(proto)
    _update_fields(proto)
    p[0] = proto


def p_header(p):
    """header : header_unit_ header
              |"""
    if len(p) == 1:
        p[0] = []
    else:
        p[0] = [p[1]] + p[2]


def p_header_unit_(p):
    """header_unit_ : header_unit ';'
                    | header_unit"""
    p[0] = p[1]


def p_header_unit(p):
    """header_unit : syntax
                   | package"""
    p[0] = p[1]


def p_syntax(p):
    """syntax : SYNTAX '=' LITERAL"""
    if p[3] != 'proto3':
        raise ProtoGrammarError(
            'grammar error at line {}: syntax must be proto3'.format(p.lineno))
    p[0] = (HeaderType.SYNTAX, p[3])


def p_package(p):
    """package : PACKAGE IDENTIFIER"""
    p[0] = (HeaderType.PACKAGE, p[2])


def p_definition(p):
    """definition : definition definition_unit_
                  |"""
    if len(p) == 1:
        p[0] = []
    else:
        p[0] = [p[2]] + p[1]


def p_definition_unit_(p):
    """definition_unit_ : definition_unit ';'
                        | definition_unit"""
    p[0] = p[1]


def p_definition_unit(p):
    """definition_unit : service
                       | message"""
    p[0] = p[1]


def p_message(p):
    """message : seen_message '{' field_seq '}'"""
    message = Message(p[1])
    for index, field in enumerate(p[3]):
        optional, type_, name, number = field

        if name in message.fields:
            raise ProtoGrammarError(
                'grammar error: message {} is already defined at line {}'.format(
                    name, p.lineno))

        label = 1
        if optional == 'repeated':
            label = 3

        message_type = None
        if type_[0] == FieldType.BASE:
            class_ = field_map[type_[1]]
            field = class_(name, index, number, label,
                           message_type=message_type)
            if label == 3:
                field.default_value = []
        elif type_[0] == FieldType.MAP:
            label = 3
            field = MapField(name, index, number, label,
                             message_type=message_type,
                             key_type=type_[2],
                             value_type=type_[3])
            field.default_value = []
        else:
            field = MessageField(name, index, number, label,
                                 message_type=type_[1])
        message.fields[name] = field

    p[0] = message


def p_seen_message(p):
    """seen_message : MESSAGE IDENTIFIER"""
    p[0] = p[2]


def p_field_seq(p):
    """field_seq : field ';' field_seq
                 |"""
    if len(p) == 1:
        p[0] = []
    else:
        p[0] = [p[1]] + p[3]


def p_field(p):
    """field : field_req field_type IDENTIFIER '=' INTCONSTANT"""
    p[0] = (p[1], p[2], p[3], p[5])


def p_field_req(p):
    """field_req : OPTIONAL
                 | REPEATED
                 |"""
    if len(p) == 2 and p[1] == 'repeated':
        p[0] = 'repeated'
    else:
        p[0] = 'optional'


def p_field_type(p):
    """field_type : ref_type
                  |  definition_type"""
    p[0] = p[1]


def p_ref_type(p):
    """ref_type : IDENTIFIER"""
    p[0] = (FieldType.REF, p[1])


def p_definition_type(p):
    """definition_type : base_type
                       | container_type"""
    p[0] = p[1]


def p_base_type(p):
    """base_type : BOOL
                 | STRING
                 | BYTES
                 | DOUBLE
                 | FLOAT
                 | INT32
                 | INT64
                 | UINT32
                 | UINT64
                 | SINT32
                 | SINT64
                 | FIXED32
                 | FIXED64
                 | SFIXED32
                 | SFIXED64"""
    p[0] = (FieldType.BASE, p[1])


def p_container_type(p):
    """container_type : map_type"""
    p[0] = p[1]


def p_map_type(p):
    """map_type : MAP '<' base_type ',' base_type '>'
               | MAP '<' base_type ',' ref_type '>'"""
    if p[3] == 'bytes':
        raise ProtoGrammarError(
            'grammar error at line {}: key type cannot be bytes'.format(p.lineno))
    p[0] = (FieldType.MAP, p[1], p[3], p[5])


def p_service(p):
    """service : SERVICE IDENTIFIER '{' func_seq '}'"""
    service = Service(p[2])
    for method in p[4]:
        if method.name in service.methods:
            raise ProtoGrammarError(
                'grammar error: method {} is already defined at line {}'.format(
                    method.name, p.lineno))
        service.methods[method.name] = method
    p[0] = service


def p_func_seq(p):
    """func_seq : func func_seq
                | func ';' func_seq
                |"""
    p_len = len(p)
    if p_len == 1:
        p[0] = []
    elif p_len == 3:
        p[0] = [p[1]] + p[2]
    else:
        p[0] = [p[1]] + p[3]


def p_func(p):
    """func : RPC IDENTIFIER '(' IDENTIFIER ')' RETURNS '(' IDENTIFIER ')' '{' '}'"""
    p[0] = ServiceMethod(p[2], p[4], p[8])


def _check_methods(proto):
    """检查serivce中定义方法的传参能否解析

    Args:
        proto (Protobuf): 解析得到的对象
    """
    for _, service in proto.services.items():
        for method_name, method in service.methods.items():
            if method.request_type in proto.messages and \
                            method.response_type in proto.messages:
                continue
            raise ProtoGrammarError(
                'grammar error: {} params not defined'.format(method_name,))


def _update_fields(proto):
    """更新map中的自定义类型以及检查嵌套message类型是否存在

    Args:
        proto (Protobuf): 解析得到的对象
    """
    for _, message in proto.messages.items():
        for field_name, field in message.fields.items():
            if isinstance(field, MessageField):
                if field_name not in proto.messages:
                    raise ProtoGrammarError(
                        'grammar error: {} not defined'.format(field_name))

                field.message_type = proto.messages[field_name]
            elif isinstance(field, MapField):
                key_type, key_name = field.key_type
                val_type, val_name = field.value_type

                if val_type == FieldType.REF:
                    if val_name not in proto.messages:
                        raise ProtoGrammarError(
                            'grammar error: {} not defined'.format(field_name))

                    field.value_type = MessageField(
                        val_name, 1, 1, 1, proto.messages[val_name])
                else:
                    class_ = field_map[val_name]
                    field.value_type = class_('value', 1, 1, 1)

                class_ = field_map[key_name]
                field.key_type = class_('key', 0, 0, 1)



import redis
from datetime import datetime


class RedisCache(object):
    """ Redis操作函数 """

    def connect(self, host, port, db, password):
        pool = redis.ConnectionPool(host=host,
                                    port=port,
                                    db=db,
                                    password=password,
                                    decode_responses=True)
        self.r = redis.Redis(connection_pool=pool)

    def set(self, key, value, expired=None):
        if isinstance(value, str):
            if expired and isinstance(expired, datetime):
                dif = expired - datetime.now()
                seconds = int(dif.total_seconds())
                if seconds < 0:
                    seconds = 0
                expired = seconds
            else:
                expired = None

            self.r.set(key, value, ex=expired)

    def get(self, key):
        return self.r.get(key)

    def ttl(self, key):
        return self.r.ttl(key)

    def expire(self, key, time):
        return self.r.expire(key, time)

    def sadd(self, name, value):
        return self.r.sadd(name, value)

    def smembers(self, name):
        return self.r.smembers(name)

    def delete(self, name):
        self.r.delete(name)



import logging
import sys
import json
from flask_douwa import routes
from flask_douwa.rpc.generator_id import GeneratorRpc
from flask_douwa.rpc.demo import ProxyGetRpc
from flask_douwa.cache import RedisCache
from flask_douwa.rpc_client.greeter_client import run
import _thread as thread
from flask_douwa import kafka_broker
import ssl
import socket
from kafka import KafkaProducer, KafkaConsumer,TopicPartition
from kafka.errors import KafkaError



"""
start 从阿里demo抄的
"""
context = ssl.create_default_context()
context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
context.verify_mode = ssl.CERT_REQUIRED

# context.check_hostname = True
context.load_verify_locations("ca-cert")

"""
end 从阿里demo抄的
"""


redis = RedisCache()
logger = logging.getLogger(__name__)


class Douwa(object):
    client_access = list()

    def __init__(self, app=None):
        self.getid = None
        self.kafka_callback = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        host = app.config.get("GENERATORID_IP", None)
        r_host = app.config.get("REDIS_HOST", None)
        r_port = app.config.get("REDIS_PORT", None)
        r_db = app.config.get("REDIS_DB", None)
        r_pwd = app.config.get("REDIS_PWD", None)
        key_url = app.config.get("KEY_URL")

        if not host:
            logger.error("GENERATORID_IP:随机id生成器服务器地址没有配置")
            sys.exit(0)
        if not r_host:
            logger.error("REDIS_HOST:没有配置REDIS HOST")
            sys.exit(0)
        if not r_port:
            logger.error("REDIS_PORT:没有配置REDIS PORT")
            sys.exit(0)
        if not r_db:
            logger.error("REDIS_DB:没有配置REDIS DATABASE NAME")
            sys.exit(0)
        if not key_url:
            logger.error("KEY_URL: 没有配置KEY_URL")
            sys.exit(0)

        # 随机生成id
        rpc_register = routes.register
        proto = rpc_register(GeneratorRpc)
        self.getid = ProxyGetRpc(host, proto[0], GeneratorRpc.name)


        # REDIS连接
        redis.connect(r_host, r_port, r_db, r_pwd)

        # access_token 设置永久1个月token时间
        Douwa.client_access = app.config.get("ACCESS_TOKEN", list())

        # kafka 设置
        self.consumer_pid = app.config.get("CONSUMER_PID")
        self.conf = dict()
        self.conf["bootstrap.servers"] = app.config.get("KAFKA_HOST")
        self.conf['retries'] = app.config.get("KAFKA_RETRIES", 5)

        # self.conf['sasl.mechanisms'] = "PLAIN"
        # self.conf['security.protocol'] = 'SASL_SSL'
        # self.conf["ssl.ca.location"] = app.config.get("SSL_PATH")
        # self.conf['sasl.username'] = app.config.get("KAFKA_USERNAME")
        # self.conf['sasl.password'] = app.config.get("KAFKA_PASSWORD")

        self.topic = app.config.get("TOPIC", None)
        """start kafka相关的东西"""

        if self.topic and self.conf["bootstrap.servers"]:
            from flask_douwa import kafka_broker

            if not self.kafka_callback:
                logging.error("启动kafka错误 没有监视函数")
                return
            logging.info("启动kafka consumter 服务")
            thread.start_new(kafka_broker.read_notification_from_ceilometer_over_kafka,
                             (self.conf, self.topic, self.consumer_pid, self.kafka_callback))

        else:
            """使用的是kafka-python 包"""
            default_setting_producer = {
                "sasl_mechanism": "PLAIN",
                "ssl_context": context,
                "security_protocol": "SASL_SSL",
                "retries": 5
            }
            default_setting_consumer = {
                "sasl_mechanism": "PLAIN",
                "ssl_context": context,
                "security_protocol": "SASL_SSL"
            }

            self.kafka_setting_producer = app.config.get("KAFKA_SETTING_PORDUCER")
            if self.kafka_setting_producer:
                self.topic_name_producer = self.kafka_setting_producer.pop("topic_name")
                self.kafka_setting_producer.update(default_setting_producer)
                self.producer = KafkaProducer(**self.kafka_setting_producer)
                self.producer.partitions_for(self.topic_name_producer)

            self.kafka_setting_consumer = app.config.get("KAFKA_SETTING_CONSUMER")
            if self.kafka_setting_consumer:
                self.topic_name_consumer = self.kafka_setting_consumer.pop("topic_name")
                tp = TopicPartition(self.topic_name_consumer, 1)
                self.kafka_setting_consumer.update(default_setting_consumer)
                self.consumer = KafkaConsumer(**self.kafka_setting_consumer)
                self.consumer.assign([tp])
            """使用的是kafka-python 包"""

    def send_message(self, message,partition=1):
        if self.topic and self.conf["bootstrap.servers"]:
            kafka_broker.send_message(self.conf, self.topic, message)
            return True
        elif self.kafka_setting_producer:
            try:
                future = self.producer.send(self.topic_name_producer, message,partition = partition)
                future.get()
                return True
            except KafkaError as e:
                logger.error(e)
                return False
        else:
            logger.error("kafka配置错误")
            return False

    def generator_id(self):
        if self.getid:
            return self.getid.GeneratorId()
        else:
            raise Exception("没有初始化")

    def get_message(self, data, arg):
        if arg == "user" or arg == "group":
            return run(data, arg)
        user = None
        if data.get('key'):
            key_prefix = "douwa:key:"
            key = key_prefix + data.get('key')
            user_cached = redis.get(key)
            if user_cached:
                try:
                    user = json.loads(user_cached)
                    # print(user)
                except Exception as e:
                    user = None
                    raise Exception(e)
            if user:
                PERMISSION_PREFIX = "duowa:permission:"
                userrole = set(user["roles"])
                key = PERMISSION_PREFIX + "角色与权限"
                dd = redis.smembers(key)
                if userrole & dd:
                    return run(data, arg)
                else:
                    error_str = "没有操作权限!"
                    return {'401': error_str}
            else:
                error_str = "用户没有登录!"
                return {'401': error_str}

        elif data.get('token'):
            prefix = "douwa:token:"
            key = prefix + data.get('token')
            user_cached = redis.get(key)
            if user_cached:
                try:
                    user = json.loads(user_cached)
                    # print(user)
                except Exception as e:
                    user = None
                    raise Exception(e)
            if user:
                PERMISSION_PREFIX = "duowa:permission:"
                userrole = set(user["roles"])
                key = PERMISSION_PREFIX + "角色与权限"
                dd = redis.smembers(key)
                if userrole & dd:
                    return run(data, arg)
                else:
                    error_str = "没有操作权限!"
                    return {'401': error_str}
            else:
                error_str = "用户没有登录!"
                return {'401': error_str}


    def bind(self):
        def wrapper(func):
            if self.kafka_callback:
                raise AttributeError('有存在的topic')
            self.kafka_callback = func
        return wrapper



# -*- coding: utf-8 -*-


#以字符串的方式传入参数，解析该字符串参数，让它变成可以调用的函数或者方法
def load_class(name):
    components = name.split('.')
    module_path = '.'.join(components[:-1])
    class_name = components[-1]
    mod = __import__(module_path, fromlist=[class_name])
    klass = getattr(mod, class_name)
    return klass


def expand(template, count, params={}, value=None, expanders=[]):
    for expander_class in expanders:
        klass = load_class(expander_class)
        expander = klass(template, count, params, value)
        value = expander.expand()
    return value



class Field(object):
    def __init__(self,
               repeated=False,
               default=None):
        self.repeated = repeated
        self.default = default


class IntegerField(Field):
    name = "int64"


class FloatField(Field):
    name = "double"


class BooleanField(Field):
    name = "bool"


class BytesField(Field):
    name = "bytes"


class StringField(Field):
    name = "string"



# -*- coding=utf8 -*-
"""异常定义"""


class ProtoParserError(Exception):
    """解析错误基类"""
    pass


class ProtoLexerError(ProtoParserError):
    """词法解析错误"""
    pass


class ProtoGrammarError(ProtoParserError):
    """语法解析错误"""
    pass



# -*- coding=utf8 -*-
"""词法解析"""
from .exc import ProtoLexerError


literals = '.;,={}<>[]()'

# enum packed singular import reserved option oneof等暂不支持
keywords = (
    'syntax',
    'package',
    'message',
    'optional',
    'repeated',
    'double',
    'float',
    'int32',
    'int64',
    'uint32',
    'uint64',
    'sint32',
    'sint64',
    'fixed32',
    'fixed64',
    'sfixed32',
    'sfixed64',
    'bool',
    'string',
    'bytes',
    'map',
    'service',
    'rpc',
    'returns'
)

tokens = (
    'INTCONSTANT',
    'LITERAL',
    'IDENTIFIER',
) + tuple(map(lambda kw: kw.upper(), keywords))

t_ignore = ' \t\r'


def t_error(t):
    raise ProtoLexerError('illegal character {} at line {}'.format(
        t.value[0], t.lineno))


def t_newline(t):
    r'\n+'
    t.lexer.lineno += len(t.value)


def t_ignore_COMMENT(t):
    r'\/\/[^\n]*'


def t_HEXCONSTANT(t):
    r'0x[0-9A-Fa-f]+'
    t.value = int(t.value, 16)
    t.type = 'INTCONSTANT'
    return t


def t_INTCONSTANT(t):
    r'[+-]?[0-9]+'
    t.value = int(t.value)
    return t


def t_LITERAL(t):
    r'(\"([^\\\n]|(\\.))*?\")|\'([^\\\n]|(\\.))*?\''
    s = t.value[1:-1]
    maps = {
        't': '\t',
        'r': '\r',
        'n': '\n',
        '\\': '\\',
        '\'': '\'',
        '"': '\"'
    }
    i = 0
    length = len(s)
    val = ''
    while i < length:
        if s[i] == '\\':
            i += 1
            if s[i] in maps:
                val += maps[s[i]]
            else:
                msg = 'unexpected escaping character: {}'.format(s[i])
                raise ProtoLexerError(msg)
        else:
            val += s[i]

        i += 1

    t.value = val
    return t


def t_IDENTIFIER(t):
    r'[a-zA-Z_](\.[a-zA-Z_0-9]|[a-zA-Z_0-9])*'

    if t.value in keywords:
        t.type = t.value.upper()
    return t



from flask_douwa.protoparser import make_client, load_byte
from flask_douwa.rpc.generator_id import GeneratorRpc
import grpc
from flask_douwa import routes


class ProxyGetRpc(object):
    def __init__(self, host, data, name):
        self.client = make_client(load_byte(data))
        self.channel = grpc.insecure_channel(host)
        self.stub = getattr(self.client,name+"Stub")(self.channel)

    def __getattr__(self, name):
        fun = getattr(self.stub, name)
        def run(**kwargs):
            request = getattr(self.client,name+"Request")(**kwargs)
            return fun(request)._result
        return run




import json
import requests

from functools import wraps
from flask import request, g, current_app
from flask_douwa import redis, Douwa
from flask_douwa import reqparse

TOKEN_PREFIX = "douwa:token:"
TOKEN_EXPIRES = 3600
TOKEN_EXPIRES2 = 3600 * 12 * 30
PERMISSION_PREFIX = "duowa:permission:"
KEY_PREFIX = 'douwa:key:'


def authorization(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        prefix = TOKEN_PREFIX
        key_prefix = KEY_PREFIX
        ttl = TOKEN_EXPIRES

        user = None
        token = None
        if 'Authorization' in request.headers:
            token = request.headers.get('Authorization')[7:]
        elif 'AuthKey' in request.headers:
            header_key = request.headers.get('AuthKey')[7:]
            if header_key:
                key = key_prefix + header_key
                # print(redis_key)
                user_cached = redis.get(key)
                if not user_cached:
                    # 发送激活key的请求
                    key_url = current_app.config.get('KEY_URL')
                    headers = {'AuthKey': 'bearer ' + header_key}
                    res = requests.get(url=key_url, headers=headers)
                    # print(res.status_code)
                    if not res.status_code == 200:
                        reqparse.abort(400, message='KEY配置错误')

                try:
                    user_cached = redis.get(key)
                    user = json.loads(user_cached)
                except Exception as e:
                    user = None
                    raise Exception(e)

        else:
            parserd = reqparse.RequestParser()
            parserd.add_argument('access_token', "token", type=str, location='args')
            urlargs = parserd.parse_args()
            if "access_token" in urlargs:
                token = urlargs["access_token"]
            else:
                token = None
        if token:
            key = prefix + token
            user_cached = redis.get(key)
            if user_cached:
                try:
                    user = json.loads(user_cached)
                except Exception as e:
                    user = None
                    raise Exception(e)
        if not user:
            error_str = "用户没有登录!"
            reqparse.abort(401, message=error_str)
        if token not in Douwa.client_access:
            expired = redis.ttl(key)
            if expired:
                redis.expire(key, ttl)
        else:
            redis.expire(key, TOKEN_EXPIRES2)

        g.user = user
        return f(*args, **kwargs)

    return decorated_function


def permission(name):
    def wrapper(func):
        @wraps(func)
        def inner(*args, **kwargs):
            userrole = set(g.user["roles"])
            key = PERMISSION_PREFIX + name
            dd = redis.smembers(key)
            if userrole & dd:
                return func(*args, **kwargs)
            else:
                error_str = "没有操作权限!"
                reqparse.abort(401, message=error_str)

        return inner

    return wrapper


def error(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        try:
            data = f(*args, **kwargs)
            return data
        except Exception as e:
            if hasattr(e, "data"):
                return json.dumps(e.data, ensure_ascii=False)
            else:
                raise e

    return decorated_function



import json

import kafka
import logging as LOG


class KafkaBrokerPublisher():
    def __init__(self, host, port, topic):
        self.kafka_client = None
        self.kafka_server = None

        self.host = host
        self.port = port
        self.topic = topic

        try:
            self._get_client()
            self._get_server()
        except Exception as e:
            LOG.exception("Failed to connect to Kafka service: %s", e)

    def _get_client(self):
        if not self.kafka_client:
            self.kafka_client = kafka.KafkaClient(
                "%s:%s" % (self.host, self.port))
            self.kafka_producer = kafka.SimpleProducer(self.kafka_client)

    def _get_server(self):
        if not self.kafka_server:
           self.kafka_server = kafka.KafkaClient(
                "%s:%s" % (self.host, self.port))
           self.kafka_consumer = kafka.KafkaConsumer(self.topic,bootstrap_servers = ["%s:%s" % (self.host, self.port)])

    def send(self, data):
        try:
            self.kafka_producer.send_messages(
                self.topic, json.dumps(data).encode("utf-8"))
        except Exception as e:
            LOG.exception(("Failed to send sample data: %s"), e)
            raise



import msgpack
import confluent_kafka
from confluent_kafka import KafkaError, KafkaException

api_version_request = True


def read_notification_from_ceilometer_over_kafka(conf, topic, pid, callbacks):
    print('Using confluent_kafka module version %s (0x%x)' % confluent_kafka.version())
    print('Using librdkafka version %s (0x%x)' % confluent_kafka.libversion())

    default_conf = {'group.id': pid,
            'session.timeout.ms': 6000,
            'enable.auto.commit': False,
            'client.id':'kafka-python-1.3.5',
            'ssl.key.password':'KafkaOnsClient',
            'ssl.certificate.location':'/Users/iye/project/oauth/oauth/kafka.client.truststore.jks',
            'default.topic.config': {
                 'auto.offset.reset': 'earliest'
            }}
    conf.update(default_conf)
    consumer = confluent_kafka.Consumer(**conf)
    consumer.subscribe(topic)
    try:
        while True:
            msg = consumer.poll(timeout=10)
            if msg is None:
                continue
            if msg.error():
                if msg.error().code() != KafkaError._PARTITION_EOF:
                    raise KafkaException(msg.error())
            else:
                data = None
                if msg.value():
                    data = msgpack.loads(msg.value(), encoding='utf-8')
                    if data:
                        t = msg.topic()
                        if t and t in callbacks:
                            commit = callbacks[msg.topic()](data)
                            if commit:
                                consumer.commit(msg, async=True)
    except KeyboardInterrupt:
        print('%% Aborted by user\n')
    consumer.close()


def send_message(conf, topic, message):

        default_conf = {'api.version.request': api_version_request,
            'client.id':'kafka-python-1.3.5',
            'ssl.key.password':'KafkaOnsClient',
            'ssl.certificate.location':'/Users/iye/project/oauth/oauth/kafka.client.truststore.jks',
                        'default.topic.config': {'produce.offset.report': True}
                        }
        conf.update(default_conf)
        p = confluent_kafka.Producer(**conf)

        try:
            p.produce(topic, value=msgpack.dumps(message, use_bin_type=True))
            p.poll(0)
        except BufferError as e:
            print('%% Local producer queue is full (%d messages awaiting delivery): try again\n' %
                            len(p))
        p.flush()


if __name__ == '__main__':
    bootstrap_servers = "kafka-ons-internet.aliyun.com:8080"
    topic = 'ptopic'
    send_message(conf, topic, {"hello":"world"})
    print(get_message(topic))
